require 'dump_truck/version'

require 'colorize'

require 'dump_truck/configuration'
require 'dump_truck/database_configuration'
require 'dump_truck/schema_configuration'
require 'dump_truck/table_configuration'

require 'dump_truck/target'
require 'dump_truck/truck'
require 'dump_truck/loggable_truck'
require 'dump_truck/mysql'

class DumpTruck
  attr_reader :config

  def initialize(&block)
    @config = Configuration.new(&block)
  end

  def dump
    config.each_database do |db_config|
      dump_database(db_config)
    end
  end

private

  def dump_database(db_config)
    db_config.each_schema.map do |schema_config|
      truck(db_config, schema_config)
    end.map do |truck|
      Thread.new(truck){|t| t.dump}
    end.each do |thread|
      thread.join
    end
  end

  def truck(db_config, schema_config)
    translator = translator_for(db_config.type)
    client = client_for(db_config.type, db_config, schema_config)

    LoggableTruck.new(schema_config, client, translator, config.logger)
  end

  def translator_for(type)
    case type.to_sym
    when :mysql
      Mysql::Translator.new
    else
      raise "Unknown type #{type}"
    end
  end

  def client_for(type, db_config, schema_config)
    case type.to_sym
    when :mysql
      Mysql::Client.new(db_config, schema_config)
    else
      raise "Unknown type #{type}"
    end
  end
end
