require 'logger'

class DumpTruck
  class Configuration
    attr_reader :databases

    def initialize
      @databases = []
      @logger = Logger.new($stdout)
      @logger.formatter = proc do |severity, datetime, progname, msg|
        "#{datetime.strftime('%Y-%m-%d %H:%M:%S').blue} #{msg}\n"
      end

      instance_eval(&Proc.new) if block_given?
    end

    def logger(logger = nil)
      @logger = logger || @logger
    end

    def database(type, &block)
      type = type.to_s.downcase.to_sym

      @databases << DatabaseConfiguration.new(type, &block)
    end

    def each_database
      if block_given?
        @databases.each(&Proc.new)
      else
        @databases.each
      end
    end
  end
end
