class DumpTruck
  module Mysql
    class Client
      def initialize(connection_config, schema_config)
        @ssh_user = connection_config.ssh_user
        @ssh_hostname = connection_config.ssh_hostname
        @schema = schema_config.name

        write_defaults_file(connection_config)
      end

      def tables_dump
        IO.popen(tables_dump_command, &Proc.new)
      end

      def data_dump(table_config, table)
        IO.popen(data_dump_command(table_config, table), &Proc.new)
      end

    private
      attr_reader :ssh_user, :ssh_hostname, :hostname, :user, :password, :schema, :file

      def cnf_file
        file.path
      end

      def write_defaults_file(connection_config)
        hostname = connection_config.hostname
        user = connection_config.user
        password = connection_config.password

        @file = Tempfile.new('my.cnf')
        @file.write <<-MYCNF.gsub(/^\s+/, '')
          [mysqldump]
          user = #{user}
          #{"password = #{password}" if !password.nil? && !password.empty?}
          host = #{hostname}
          hex-blob
          no-create-db
          skip-comments
          single-transaction
          complete-insert
          compress
          net_buffer_length = 1048576
        MYCNF
        @file.close
      end

      def tables_dump_command
        "#{ssh_connection} mysqldump --defaults-extra-file=#{cnf_file} --no-data --no-set-names --no-tablespaces --skip-add-drop-table --skip-add-locks --skip-set-charset #{schema}"
      end

      def data_dump_command(config, table)
        "#{ssh_connection} mysqldump --defaults-extra-file=#{cnf_file} #{data_flag(config)} #{where_flag(config)} #{schema} #{table}"
      end

      def ssh_connection
        user_string = "#{@ssh_user}@" if @ssh_user

        "ssh -C #{user_string}#{@ssh_hostname} " if @ssh_hostname
      end

      def data_flag(config)
        config.mode == :none ? '--no-data' : ''
      end

      def where_flag(config)
        config.mode == :some ? "--where='#{config.query}'" : ''
      end
    end
  end
end

