class DumpTruck
  class SchemaConfiguration
    attr_reader :name

    def initialize(name)
      @name = name
      @tables = Hash.new(TableConfiguration.new{truncate})
      @target_path = 'tmp'
      @target_name_generator = proc{|schema| schema}

      instance_eval(&Proc.new) if block_given?
    end

    def target_path(target_path = nil)
      @target_path = target_path || @target_path
    end

    def target_name
      @target_name_generator = Proc.new
    end

    def target
      File.join(@target_path, @target_name_generator.call(@name) + '.sql.gz')
    end

    def table_default
      if block_given?
        @tables.default = TableConfiguration.new(&Proc.new)
      else
        @tables.default
      end
    end

    def table(name, &block)
      name = name.to_s
      @tables[name] = TableConfiguration.new(name, &block)
    end

    def tables
      @tables.values
    end

    def table_config_for(table)
      @tables[table.to_s]
    end

    def ==(other)
      name == other.name && target_path == other.target_path && tables == other.tables
    end

    def to_s
      "<DumpTruck::SchemaConfiguration(#{name}) target:#{target} (#{tables.map(&:to_s).join(', ')})>"
    end
  end
end
