class DumpTruck
  class TableConfiguration
    attr_reader :name, :mode, :query, :obfuscators

    def initialize(name = nil)
      @name = name.to_s
      @obfuscators = Hash.new(proc{|value| value})
      @mode = :none

      instance_eval(&Proc.new) if block_given?
    end

    def truncate
      @mode = :none
    end

    def ignore
      @mode = :ignore
    end

    def keep(query = nil)
      @query = query

      if query
        @mode = :some
      else
        @mode = :all
      end
    end

    def obfuscate_default
      if block_given?
        @obfuscators.default = Proc.new
      else
        @obfuscators.default
      end
    end

    def obfuscate(column)
      column = column.to_s
      if block_given?
        @obfuscators[column] = Proc.new
      else
        @obfuscators[column] = proc{nil}
      end
    end

    def obfuscate_value(column, value, n)
      @obfuscators[column.to_s].call(value, n)
    end

    def ==(other)
      name == other.name &&
        mode == other.mode &&
        (mode != :some || query == other.query)
    end

    def to_s
      query_string = mode == :some ? ", query: #{query}" : ''
      "<DumpTruck::TableConfiguration(#{name}) mode: #{mode}#{query_string} (#{obfuscators.keys.join(', ')})>"
    end
  end
end
