require 'zlib'

class DumpTruck
  class Truck
    def initialize(schema_config, client, translator)
      @schema_config = schema_config
      @client = client
      @translator = translator
    end

    def dump
      tables = extract_tables
      dump_schema(tables)
    end

  protected

    attr_reader :schema_config, :client, :translator

    def extract_tables
      client.tables_dump do |io|
        io.each.map do |line|
          extract_table(line) if translator.table?(line)
        end.compact
      end
    end

    def dump_schema(tables)
      Target.new(schema_config.target).open do |target|
        tables.each do |table|
          config = schema_config.table_config_for(table)

          dump_data(config, table, target) unless config.mode == :ignore
        end
      end
    end

    def dump_data(config, table, target)
      client.data_dump(config, table) do |io|
        row_num = 0
        io.each do |line|
          line, row_num = translator.insert?(line) ? obfuscate(config, table, line, row_num) : [line, row_num]

          target.write(line)
        end
      end
    end

    def obfuscate(config, table, line, row_num)
      return '' if config.mode == :none

      fields, data = extract_insert(line)

      data = data.map do |datum|
        row_num += 1
        Hash[datum.map{|field, value| [field, config.obfuscate_value(field, value, row_num)]}]
      end

      [translator.generate_insert(table, fields, data), row_num]
    end

    def extract_table(line)
      translator.extract_table(line)
    end

    def extract_insert(line)
      translator.extract_insert(line)
    end
  end
end
